require "import"
import "android.widget.*"
import "android.app.*"
import "android.view.*"
import "android.os.*"
import "android.text.InputType"

local NOTES_FILE = "/sdcard/Download/notes.txt"
local EXPORT_FILE = "/sdcard/Download/exported_notes.txt"
local notes = {}

function loadNotes()
  notes = {}
  local file = io.open(NOTES_FILE, "r")
  if file then
    for line in file:lines() do
      local time, title, content = line:match("^(.-);(.-);(.*)$")
      if time and title and content then
        table.insert(notes, {time = time, title = title, content = content})
      end
    end
    file:close()
  end
end

function saveNotes()
  local file = io.open(NOTES_FILE, "w")
  for _, note in ipairs(notes) do
    file:write(note.time .. ";" .. note.title .. ";" .. note.content .. "\n")
  end
  file:close()
end

function exportNotes()
  local file = io.open(EXPORT_FILE, "w")
  if file then
    for _, note in ipairs(notes) do
      file:write("Title: " .. note.title .. "\n")
      file:write("Time: " .. note.time .. "\n")
      file:write("Content:\n" .. note.content .. "\n\n")
    end
    file:close()
    Toast.makeText(activity, "Notes exported", 1).show()
  else
    Toast.makeText(activity, "Export failed!", 1).show()
  end
end

function showMainLayout()
  layout = {
    LinearLayout,
    orientation = "vertical",
    layout_width = "match_parent",
    layout_height = "match_parent",
    {
      TextView,
      id = "header",
      text = "Professional Notebook",
      textSize = "22sp",
      gravity = "center",
      background = "#3F51B5",
      textColor = "#ffffff",
      padding = "16dp",
      layout_width = "match_parent",
      layout_height = "wrap_content"
    },
    {
      TextView,
      id = "noteCount",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      textSize = "14sp",
      gravity = "center",
      textColor = "#444444",
      padding = "8dp"
    },
    {
      ListView,
      id = "noteList",
      layout_width = "match_parent",
      layout_height = "0dp",
      layout_weight = "1"
    },
    {
      LinearLayout,
      orientation = "horizontal",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      background = "#eeeeee",
      {
        Button,
        id = "btnAdd",
        text = "New note",
        layout_width = "0dp",
        layout_height = "wrap_content",
        layout_weight = "1"
      },
      {
        Button,
        id = "btnSearch",
        text = "Search notes",
        layout_width = "0dp",
        layout_height = "wrap_content",
        layout_weight = "1"
      },
      {
        Button,
        id = "btnExport",
        text = "Export",
        layout_width = "0dp",
        layout_height = "wrap_content",
        layout_weight = "1"
      }
    }
  }

  activity.setContentView(loadlayout(layout))

  function updateList(filter)
    local items = {}
    for i, note in ipairs(notes) do
      if not filter or note.title:lower():find(filter:lower()) then
        table.insert(items, note.title)
      end
    end
    noteCount.Text = "Total Notes: " .. #notes
    noteList.setAdapter(ArrayAdapter(activity, android.R.layout.simple_list_item_1, items))
  end

  noteList.onItemClick = function(l, v, p)
    showNoteDetail(p + 1)
  end

  btnAdd.onClick = function()
    local addLayout = {
      LinearLayout,
      orientation = "vertical",
      padding = "16dp",
      {
        EditText,
        id = "titleInput",
        hint = "Title",
        layout_width = "match_parent"
      },
      {
        EditText,
        id = "contentInput",
        hint = "Content",
        layout_width = "match_parent",
        layout_height = "150dp",
        gravity = "top"
      }
    }

    AlertDialog.Builder(activity)
      .setTitle("New Note")
      .setView(loadlayout(addLayout))
      .setPositiveButton("Save", function()
        local title = titleInput.Text
        local content = contentInput.Text
        if title ~= "" and content ~= "" then
          local time = os.date("%Y-%m-%d %H:%M:%S")
          table.insert(notes, {time = time, title = title, content = content})
          saveNotes()
          updateList()
          Toast.makeText(activity, "Note saved", 1).show()
        else
          Toast.makeText(activity, "Please fill both fields", 1).show()
        end
      end)
      .setNegativeButton("Cancel", nil)
      .show()
  end

  btnSearch.onClick = function()
    local input = EditText(activity)
    input.hint = "Search by title"
    AlertDialog.Builder(activity)
      .setTitle("Search Notes")
      .setView(input)
      .setPositiveButton("Search", function()
        updateList(input.Text)
      end)
      .setNegativeButton("Cancel", nil)
      .show()
  end

  btnExport.onClick = function()
    AlertDialog.Builder(activity)
      .setTitle("Export Notes")
      .setMessage("Export all notes to a text file?")
      .setPositiveButton("Export", function()
        exportNotes()
      end)
      .setNegativeButton("Cancel", nil)
      .show()
  end

  updateList()
end

function showNoteDetail(index)
  local note = notes[index]
  local detailLayout = {
    ScrollView,
    layout_width = "match_parent",
    layout_height = "match_parent",
    {
      LinearLayout,
      orientation = "vertical",
      padding = "16dp",
      {
        TextView,
        text = "Title:\n" .. note.title,
        textSize = "18sp"
      },
      {
        TextView,
        text = "\nCreated:\n" .. note.time,
        textSize = "16sp"
      },
      {
        TextView,
        text = "\nContent:\n" .. note.content,
        textSize = "16sp"
      },
      {
        LinearLayout,
        orientation = "horizontal",
        layout_width = "match_parent",
        {
          Button,
          text = "Edit",
          layout_width = "0dp",
          layout_weight = "1",
          onClick = function()
            showEditNote(index)
          end
        },
        {
          Button,
          text = "Delete",
          layout_width = "0dp",
          layout_weight = "1",
          onClick = function()
            AlertDialog.Builder(activity)
              .setTitle("Confirm Delete")
              .setMessage("Are you sure you want to delete this note?")
              .setPositiveButton("Yes", function()
                table.remove(notes, index)
                saveNotes()
                Toast.makeText(activity, "Note deleted", 1).show()
                showMainLayout()
              end)
              .setNegativeButton("No", nil)
              .show()
          end
        }
      },
      {
        Button,
        text = "Go Back",
        layout_width = "match_parent",
        onClick = function()
          showMainLayout()
        end
      }
    }
  }

  activity.setContentView(loadlayout(detailLayout))
end

function showEditNote(index)
  local note = notes[index]
  local editLayout = {
    LinearLayout,
    orientation = "vertical",
    padding = "16dp",
    {
      EditText,
      id = "editTitle",
      hint = "Title",
      text = note.title,
      layout_width = "match_parent"
    },
    {
      EditText,
      id = "editContent",
      hint = "Content",
      text = note.content,
      layout_width = "match_parent",
      layout_height = "150dp",
      gravity = "top"
    }
  }

  AlertDialog.Builder(activity)
    .setTitle("Edit Note")
    .setView(loadlayout(editLayout))
    .setPositiveButton("Save", function()
      local newTitle = editTitle.Text
      local newContent = editContent.Text
      if newTitle ~= "" and newContent ~= "" then
        notes[index].title = newTitle
        notes[index].content = newContent
        saveNotes()
        Toast.makeText(activity, "Note updated", 1).show()
        showNoteDetail(index)
      else
        Toast.makeText(activity, "Fields cannot be empty", 1).show()
      end
    end)
    .setNegativeButton("Cancel", nil)
    .show()
end

-- Start
loadNotes()
showMainLayout()